import java.net.*;
import java.io.*;
import java.util.Properties;

/**
 * Listens on a specific port and saves the incoming data to a text file.
 * For security reasons, only one client is allowed to connect.
 * 
 * Copyright (C) 2008 Jesper Jeeninga
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * @author Jesper Jeeninga
 *
 */
public class Main {

	/**
	 * @param args Usage: sampreg <port> <server> \"<filename>\"
	 */
	public static void main(String[] args) {
		try {
			if (args.length > 1) {
				throw new Exception("Usage: sampreg <configfile>");
			}
			// Load Defaults
			String localhost = "all";
			int port = 7776;
			String directory = "scriptfiles" + System.getProperty("file.separator");
			String filename = null;
			String configfile = "config.cfg";
			String client = "localhost";
			int lines = 6;
			
			if (args.length == 1) {
				configfile = args[0];
			}
			//Load configuration 
			File configFile = new File(configfile);
			if (configFile.exists()) {
				Properties properties = new Properties();
				properties.load(new FileInputStream(configfile));
			    if (properties.containsKey("client")) {
			    	client = InetAddress.getByName(properties.getProperty("client")).getHostAddress();
			    }
			    if (properties.containsKey("lines")) {
			    	lines = Integer.parseInt(properties.getProperty("lines"));	
			    }
			    if (properties.containsKey("port")) {
			    	port = Integer.parseInt(properties.getProperty("port"));
			    }
			    if(properties.containsKey("directory")) {
			    	directory = properties.getProperty("directory") + System.getProperty("file.separator");
			    }
		    	File test = new File(directory);
		    	if(!test.exists()) throw new Exception ("Directory " + directory + " does not exist");
			    if(properties.containsKey("filename")) {
			    	filename = properties.getProperty("filename");
			    }
				if (properties.containsKey("localhost")) {
					localhost = properties.getProperty("localhost");
				}
				properties = null;
			}
			configFile = null;
		    //initialize security manager
			SampSecurityManager sm = new SampSecurityManager(client,port);
	        System.setSecurityManager(sm);
			try {
				ServerSocket serverSocket;
				if(localhost.equals("all")) {
					serverSocket = new ServerSocket(port);
				}
				else {
					serverSocket = new ServerSocket(port, 0, InetAddress.getByName(localhost));
				}
				System.out.println("Listening on : " + serverSocket.getInetAddress() + ":" + port + " for " + client);
				PrintStream clientOut = null; 
				Socket clientSocket = null;
				while(true) {
					try {
						String file = filename;
						clientSocket = serverSocket.accept();
					    clientSocket.setSoTimeout(2000); //you have 2 seconds
						BufferedReader in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
						clientOut = new PrintStream(
					             new BufferedOutputStream(clientSocket.getOutputStream(), 1024), false);
						if ( file == null ){
							// don't have a filename, so we read that first
							file = in.readLine();
							if (file.length() > 255 ) throw new Exception("File length cannot be greater then 255");
						}
						// Try to open the file
						File outFile = new File(directory + file);
						if ( file == null || outFile.exists()) {
							throw new Exception("File already exists");
						}
						// Test the file on writing
						outFile.createNewFile();
						if(!outFile.canWrite()) {
							throw new Exception("Could not write to file " + directory + filename);
						}
			            BufferedWriter out = new BufferedWriter(new FileWriter(outFile, true));
			            String line;
			            // Read all incoming data and write them to the file.
			            for(int i =0 ; i < lines ; i++ ) 
			            {
				            line = in.readLine();
				            // System.out.println("Received from "+ clientSocket.getInetAddress().getHostAddress() + " : " + line);
				            // Maybe do some sanity checks on the data here
				            out.write(line + "\n");
			            }
			            // Close file and reply success message
			            out.close();
			            clientOut.println("Success!");
			            clientOut.flush();
					} catch (Exception e) {
						if (clientOut != null) {
							// Reply error if possible
							clientOut.println("Error: " + e.getMessage());
							clientOut.flush();
						}
						// Report error
					    System.out.println("Error: " + e.getMessage());
					}
					// Close socket
					clientOut.close();
					clientSocket.close();
					clientOut = null;
					clientSocket = null;
				}
			} catch (IOException e) {
				System.out.println("Could not listen on port: " + port);
				System.exit(-1);
			}
		}
		catch (Exception e) {
			System.out.println("Error initializing: " + e.getMessage());
			System.exit(-1);
		}
	}
}